main(){
    PATH_FILE=$(pwd)
    PYTHON_PATH=python3
    ANNOGESIC_PATH=annogesic #### specify the path of annogesic
    ANNOGESIC_FOLDER=ANNOgesic_NC_009839
    FTP_SOURCE=ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/bacteria/Campylobacter_jejuni/latest_assembly_versions/GCF_000017905.1_ASM1790v1/
    SEQ_ID=NC_009839.1
    wig_folder="ANNOgesic_NC_009839/input/wigs/tex_notex"
    TEX_LIBS="$wig_folder/GSM951380_Log_81116_R1_minus_TEX_in_NC_009839_plus.wig:notex:1:a:+ \
$wig_folder/GSM951380_Log_81116_R1_minus_TEX_in_NC_009839_minus.wig:notex:1:a:- \
$wig_folder/GSM951381_Log_81116_R1_plus_TEX_in_NC_009839_plus.wig:tex:1:a:+ \
$wig_folder/GSM951381_Log_81116_R1_plus_TEX_in_NC_009839_minus.wig:tex:1:a:- \
$wig_folder/GSM951382_Log_81116_R2_minus_TEX_in_NC_009839_plus.wig:notex:1:b:+ \
$wig_folder/GSM951382_Log_81116_R2_minus_TEX_in_NC_009839_minus.wig:notex:1:b:- \
$wig_folder/GSM951383_Log_81116_R2_plus_TEX_in_NC_009839_plus.wig:tex:1:b:+ \
$wig_folder/GSM951383_Log_81116_R2_plus_TEX_in_NC_009839_minus.wig:tex:1:b:-"

##### Please remove # before the commands that you want to run
##### Please install ANNOgesic properly or use a Docker container

    set_up_analysis_folder
#    get_wig_and_read_files
#    get_input_files    
#    Optimize_TSS
#    Optimize_PS
#    TSS_prediction
#    processing_site_prediction
#    Transcript_detection
#    Terminator_prediction
#    utr_detection
#    operon_detection
#    promoter_detection
#    sRNA_detection
#    sORF_detection
#    sRNA_target
#    CircRNA_detection
#    SNP_calling
#    GO_term
#    Subcellular_localization
#    PPI_network
#    riboswitch_and_RNA_thermometer
#    crispr
#    merge_features
}


set_up_analysis_folder(){
    if ! [ -d $ANNOGESIC_FOLDER ]
    then
        $ANNOGESIC_PATH create \
	-pj $ANNOGESIC_FOLDER
    fi
}

get_wig_and_read_files(){
    #### Download and install sratoolkit. If sratoolkit has been installed, you can skip this step.
    wget http://ftp-trace.ncbi.nlm.nih.gov/sra/sdk/2.5.2/sratoolkit.2.5.2-ubuntu64.tar.gz
    tar -zxvf sratoolkit.2.5.2-ubuntu64.tar.gz
    rm sratoolkit.2.5.2-ubuntu64.tar.gz

    #### transfer SRA files to fasta files. Please specify the path of fastq_dump
    for SRA in SRR515260 SRR515261 SRR515259 SRR515258 SRR515257 SRR515256 SRR515255 SRR515254
    do
        wget ftp://ftp-trace.ncbi.nlm.nih.gov/sra/sra-instant/reads/ByStudy/sra/SRP/SRP013/SRP013869/$SRA/$SRA.sra
        ./sratoolkit.2.5.2-ubuntu64/bin/fastq-dump.2.5.2 --fasta $SRA.sra
        rm $SRA.sra
    done
    mv *.fasta $ANNOGESIC_FOLDER/input/reads

    #### Download and unzip wiggle files
    for WIG in GSM951380 GSM951381 GSM951382 GSM951383
    do
        for R in R1 R2
        do
            wget -cP $ANNOGESIC_FOLDER/input/wigs/tex_notex ftp://ftp.ncbi.nlm.nih.gov/geo/samples/GSM951nnn/$WIG/suppl/${WIG}_Log_81116_${R}_minus_TEX_in_NC_009839_minus.wig.gz
            wget -cP $ANNOGESIC_FOLDER/input/wigs/tex_notex ftp://ftp.ncbi.nlm.nih.gov/geo/samples/GSM951nnn/$WIG/suppl/${WIG}_Log_81116_${R}_minus_TEX_in_NC_009839_plus.wig.gz
            wget -cP $ANNOGESIC_FOLDER/input/wigs/tex_notex ftp://ftp.ncbi.nlm.nih.gov/geo/samples/GSM951nnn/$WIG/suppl/${WIG}_Log_81116_${R}_plus_TEX_in_NC_009839_minus.wig.gz
            wget -cP $ANNOGESIC_FOLDER/input/wigs/tex_notex ftp://ftp.ncbi.nlm.nih.gov/geo/samples/GSM951nnn/$WIG/suppl/${WIG}_Log_81116_${R}_plus_TEX_in_NC_009839_plus.wig.gz
        done
    done
    cd $ANNOGESIC_FOLDER/input/wigs/tex_notex
    gunzip GSM951380_Log_81116_R1_minus_TEX_in_NC_009839_minus.wig.gz \
           GSM951380_Log_81116_R1_minus_TEX_in_NC_009839_plus.wig.gz \
           GSM951381_Log_81116_R1_plus_TEX_in_NC_009839_minus.wig.gz \
           GSM951381_Log_81116_R1_plus_TEX_in_NC_009839_plus.wig.gz \
           GSM951382_Log_81116_R2_minus_TEX_in_NC_009839_minus.wig.gz \
           GSM951382_Log_81116_R2_minus_TEX_in_NC_009839_plus.wig.gz \
           GSM951383_Log_81116_R2_plus_TEX_in_NC_009839_minus.wig.gz \
           GSM951383_Log_81116_R2_plus_TEX_in_NC_009839_plus.wig.gz
    cd ../../../../

    #### Chromosome names of the wiggle files are not the same as fasta files.
    #### We can use replace_seq_id.py to modify them.
    wget https://raw.githubusercontent.com/Sung-Huan/ANNOgesic/master/tutorial_data/replace_seq_id.py
    python3 replace_seq_id.py -i $ANNOGESIC_FOLDER/input/wigs/tex_notex -n NC_009839.1
    rm replace_seq_id.py
}



get_input_files(){
    $ANNOGESIC_PATH \
	get_input_files \
	-F $FTP_SOURCE \
	-g \
	-f \
	-e \
	-k \
	-p \
	-r \
	-pj $ANNOGESIC_FOLDER
}

Optimize_TSS(){
    $ANNOGESIC_PATH \
        optimize_tss_ps \
        -f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -tl $TEX_LIBS \
        -cn Log -s 25 \
        -m manual_TSS/NC_009839_manual_TSS.gff \
        -le ${SEQ_ID}:200000 \
        -rt all_2 \
        -pj $ANNOGESIC_FOLDER
}

Optimize_PS(){
    $ANNOGESIC_PATH \
        optimize_tss_ps \
        -f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -tl $TEX_LIBS \
        -cn Log -s 25 \
        -m manual_PS/NC_009839_manual_processing.gff \
        -le ${SEQ_ID}:200000 \
        -rt all_2 \
	-p PS \
        -pj $ANNOGESIC_FOLDER
}

TSS_prediction(){
    $ANNOGESIC_PATH \
        tss_ps \
        -f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -tl $TEX_LIBS \
        -cn Log \
        -he 0.4 \
        -rh 0.1 \
        -fa 1.7 \
        -rf 0.2 \
        -bh 0.039 \
        -ef 1.1 \
        -pf 4.5 \
        -rt all_2 \
        -le ${SEQ_ID}:200000 \
	-p TSS \
        -m manual_TSS/NC_009839_manual_TSS.gff \
        -pj $ANNOGESIC_FOLDER
}

processing_site_prediction()
{
    $ANNOGESIC_PATH \
        tss_ps \
        -f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -tl $TEX_LIBS \
        -cn ML \
        -he 0.2 \
        -rh 0.1 \
        -fa 2.0 \
        -rf 0.5 \
        -bh 0.009 \
        -ef 1.2 
        -pf 1.5
        -rt all_2 \
        -le ${SEQ_ID}:200000 \
        -p PS \
        -m manual_PS/NC_009839_manual_processing.gff \
        -pj $ANNOGESIC_FOLDER
}

Transcript_detection(){
    $ANNOGESIC_PATH \
        transcript \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -tl $TEX_LIBS \
        -rt all_2 \
	-cf gene CDS \
        -ct $ANNOGESIC_FOLDER/output/TSSs/gffs/${SEQ_ID}_TSS.gff \
        -pj $ANNOGESIC_FOLDER
}

Terminator_prediction(){
    $ANNOGESIC_PATH \
        terminator \
        -f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -a $ANNOGESIC_FOLDER/output/transcripts/gffs/${SEQ_ID}_transcript.gff \
        -tl $TEX_LIBS \
        -rt all_2 \
        -pj $ANNOGESIC_FOLDER
}

utr_detection(){
    $ANNOGESIC_PATH \
        utr \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -t $ANNOGESIC_FOLDER/output/TSSs/gffs/${SEQ_ID}_TSS.gff \
        -a $ANNOGESIC_FOLDER/output/transcripts/gffs/${SEQ_ID}_transcript.gff \
        -e $ANNOGESIC_FOLDER/output/terminators/gffs/best_candidates/${SEQ_ID}_term.gff \
        -pj $ANNOGESIC_FOLDER
}

operon_detection(){
    $ANNOGESIC_PATH \
        operon \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -t $ANNOGESIC_FOLDER/output/TSSs/gffs/${SEQ_ID}_TSS.gff \
        -a $ANNOGESIC_FOLDER/output/transcripts/gffs/${SEQ_ID}_transcript.gff \
        -u5 $ANNOGESIC_FOLDER/output/UTRs/5UTRs/gffs/${SEQ_ID}_5UTR.gff \
        -u3 $ANNOGESIC_FOLDER/output/UTRs/3UTRs/gffs/${SEQ_ID}_3UTR.gff \
        -e $ANNOGESIC_FOLDER/output/terminators/gffs/best_candidates/${SEQ_ID}_term.gff \
        -pj $ANNOGESIC_FOLDER
}

promoter_detection(){
    #### If MEME or GLAM2 cannot run in parallels, please install MPI (OpenMPI or MPICH).
    #### Otherwise, please remove -pl 10.
    $ANNOGESIC_PATH \
        promoter \
        -t $ANNOGESIC_FOLDER/output/TSSs/gffs/${SEQ_ID}_TSS.gff \
        -f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -w 45 \
        -pl 10 \
        -pj $ANNOGESIC_FOLDER
}

sRNA_detection(){
    #### If you have no sRNA database, you can use BSRD.
    wget -cP $ANNOGESIC_FOLDER/input/databases/ https://raw.githubusercontent.com/Sung-Huan/ANNOgesic/master/database/sRNA_database_BSRD.fa
    #### If you have no nr databse, please download it.
    wget -cP $ANNOGESIC_FOLDER/input/databases/ ftp://ftp.ncbi.nih.gov/blast/db/FASTA/nr.gz
    gunzip $ANNOGESIC_FOLDER/input/databases/nr.gz
    mv $ANNOGESIC_FOLDER/input/databases/nr $ANNOGESIC_FOLDER/input/databases/nr.fa


    $ANNOGESIC_PATH \
        srna \
        -d tss blast_srna sec_str blast_nr \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -t $ANNOGESIC_FOLDER/output/TSSs/gffs/${SEQ_ID}_TSS.gff \
        -p $ANNOGESIC_FOLDER/output/processing_sites/gffs/${SEQ_ID}_processing.gff \
        -a $ANNOGESIC_FOLDER/output/transcripts/gffs/${SEQ_ID}_transcript.gff \
        -f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -e $ANNOGESIC_FOLDER/output/terminators/gffs/best_candidates/${SEQ_ID}_term.gff \
        -pt $ANNOGESIC_FOLDER/output/promoters/${SEQ_ID}/MEME/promoter_motifs_${SEQ_ID}_allgenome_all_types_45_nt/meme.csv \
        -pn MOTIF_1 \
        -m \
        -u \
	-cs \
        -sf \
        -nf \
        -nd $ANNOGESIC_FOLDER/input/databases/nr \
	-sd $ANNOGESIC_FOLDER/input/databases/sRNA_database_BSRD \
	-tl $TEX_LIBS \
        -rt all_2 \
        -pj $ANNOGESIC_FOLDER
}

sORF_detection(){
    $ANNOGESIC_PATH \
        sorf \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -t $ANNOGESIC_FOLDER/output/TSSs/gffs/${SEQ_ID}_TSS.gff \
        -a $ANNOGESIC_FOLDER/output/transcripts/gffs/${SEQ_ID}_transcript.gff \
	-f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -s $ANNOGESIC_FOLDER/output/sRNAs/gffs/best_candidates/${SEQ_ID}_sRNA.gff \
        -tl $TEX_LIBS \
        -rt all_2 -u \
        -pj $ANNOGESIC_FOLDER
}

sRNA_target(){
    $ANNOGESIC_PATH \
        srna_target \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -s $ANNOGESIC_FOLDER/output/sRNAs/gffs/best_candidates/${SEQ_ID}_sRNA.gff \
        -q all \
        -p IntaRNA RNAplex RNAup \
	-mi H \
        -pj $ANNOGESIC_FOLDER
}

CircRNA_detection(){
    READ_FILES=$ANNOGESIC_FOLDER/input/reads/SRR515254.fasta,\
$ANNOGESIC_FOLDER/input/reads/SRR515255.fasta,\
$ANNOGESIC_FOLDER/input/reads/SRR515256.fasta,\
$ANNOGESIC_FOLDER/input/reads/SRR515257.fasta,\
$ANNOGESIC_FOLDER/input/reads/SRR515258.fasta,\
$ANNOGESIC_FOLDER/input/reads/SRR515259.fasta,\
$ANNOGESIC_FOLDER/input/reads/SRR515260.fasta,\
$ANNOGESIC_FOLDER/input/reads/SRR515261.fasta

    $ANNOGESIC_PATH \
        circrna \
	-f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -p 10 \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -rp all_samples:$READ_FILES \
        -pj $ANNOGESIC_FOLDER
}

SNP_calling(){
    BAM_FILES=$ANNOGESIC_FOLDER/output/circRNAs/segemehl_alignment_files/${SEQ_ID}/SRR515254_${SEQ_ID}.bam,\
$ANNOGESIC_FOLDER/output/circRNAs/segemehl_alignment_files/${SEQ_ID}/SRR515255_${SEQ_ID}.bam,\
$ANNOGESIC_FOLDER/output/circRNAs/segemehl_alignment_files/${SEQ_ID}/SRR515256_${SEQ_ID}.bam,\
$ANNOGESIC_FOLDER/output/circRNAs/segemehl_alignment_files/${SEQ_ID}/SRR515257_${SEQ_ID}.bam,\
$ANNOGESIC_FOLDER/output/circRNAs/segemehl_alignment_files/${SEQ_ID}/SRR515258_${SEQ_ID}.bam,\
$ANNOGESIC_FOLDER/output/circRNAs/segemehl_alignment_files/${SEQ_ID}/SRR515259_${SEQ_ID}.bam,\
$ANNOGESIC_FOLDER/output/circRNAs/segemehl_alignment_files/${SEQ_ID}/SRR515260_${SEQ_ID}.bam,\
$ANNOGESIC_FOLDER/output/circRNAs/segemehl_alignment_files/${SEQ_ID}/SRR515261_${SEQ_ID}.bam

    $ANNOGESIC_PATH \
         snp \
	-t reference genome \
	-p with_BAQ without_BAQ extend_BAQ \
	-b all_samples:$BAM_FILES \
	-f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
	-pj $ANNOGESIC_FOLDER
}

GO_term(){
    #### If you have no goslim_generic.obo, go.obo, and idmapping_selected.tab, please download them.
    wget -cP $ANNOGESIC_FOLDER/input/databases http://www.geneontology.org/ontology/subsets/goslim_generic.obo
    wget -cP $ANNOGESIC_FOLDER/input/databases http://geneontology.org/ontology/go.obo
    wget -cP $ANNOGESIC_FOLDER/input/databases ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/idmapping/idmapping_selected.tab.gz
    gunzip $ANNOGESIC_FOLDER/input/databases/idmapping_selected.tab.gz

    $ANNOGESIC_PATH \
        go_term \
	-g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
	-a $ANNOGESIC_FOLDER/output/transcripts/gffs/${SEQ_ID}_transcript.gff \
        -go $ANNOGESIC_FOLDER/input/databases/go.obo \
        -gs $ANNOGESIC_FOLDER/input/databases/goslim_generic.obo \
        -u $ANNOGESIC_FOLDER/input/databases/idmapping_selected.tab \
	-pj $ANNOGESIC_FOLDER
}

Subcellular_localization(){
    $ANNOGESIC_PATH \
        localization \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -a $ANNOGESIC_FOLDER/output/transcripts/gffs/${SEQ_ID}_transcript.gff \
        -b negative \
        -pj $ANNOGESIC_FOLDER
}

PPI_network(){
    #### If you have no species.$Version.txt of STRING database, please download it.
    wget -cP $ANNOGESIC_FOLDER/input/databases https://string-db.org/download/species.v10.5.txt


    $ANNOGESIC_PATH \
        ppi_network \
	-s ${SEQ_ID}.gff:${SEQ_ID}:'Campylobacter jejuni 81176':'Campylobacter' \
	-g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
	-d $ANNOGESIC_FOLDER/input/databases/species.v10.5.txt \
	-q all \
	-n \
	-pj $ANNOGESIC_FOLDER
}

riboswitch_and_RNA_thermometer(){
    #### You can download the riboswitch and RNA thermometer ID list of Rfam from our Git repository.
    wget -cP $ANNOGESIC_FOLDER/input/riboswitch_ID_file/ https://raw.githubusercontent.com/Sung-Huan/ANNOgesic/master/database/Rfam_riboswitch_ID.csv
    wget -cP $ANNOGESIC_FOLDER/input/RNA_thermometer_ID_file/ https://raw.githubusercontent.com/Sung-Huan/ANNOgesic/master/database/Rfam_RNA_thermometer_ID.csv
    #### If you have no Rfam database, please download it.
    wget -cP $ANNOGESIC_FOLDER/input/databases ftp://ftp.ebi.ac.uk/pub/databases/Rfam/12.0/Rfam.tar.gz
    cd $ANNOGESIC_FOLDER/input/databases
    tar -zxvf Rfam.tar.gz
    rm Rfam.tar.gz
    cd ../../../


    $ANNOGESIC_PATH \
        riboswitch_thermometer \
	-g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
	-f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
	-ri $ANNOGESIC_FOLDER/input/riboswitch_ID_file/Rfam_riboswitch_ID.csv \
	-ti $ANNOGESIC_FOLDER/input/RNA_thermometer_ID_file/Rfam_RNA_thermometer_ID.csv \
	-R $ANNOGESIC_FOLDER/input/databases/CMs/Rfam.cm \
	-a $ANNOGESIC_FOLDER/output/transcripts/gffs/${SEQ_ID}_transcript.gff \
	-t $ANNOGESIC_FOLDER/output/TSSs/gffs/${SEQ_ID}_TSS.gff \
	-pj $ANNOGESIC_FOLDER
}

crispr(){
    $ANNOGESIC_PATH \
        crispr \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -pj $ANNOGESIC_FOLDER
}

merge_features(){
    ALL_FEATURES="$ANNOGESIC_FOLDER/output/TSSs/gffs/${SEQ_ID}_TSS.gff \
                  $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
                  $ANNOGESIC_FOLDER/output/UTRs/5UTRs/gffs/${SEQ_ID}_5UTR.gff \
                  $ANNOGESIC_FOLDER/output/UTRs/3UTRs/gffs/${SEQ_ID}_3UTR.gff \
                  $ANNOGESIC_FOLDER/output/terminators/gffs/best_candidates/${SEQ_ID}_term.gff \
                  $ANNOGESIC_FOLDER/output/processing_sites/gffs/${SEQ_ID}_processing.gff \
                  $ANNOGESIC_FOLDER/output/sRNAs/gffs/best_candidates/${SEQ_ID}_sRNA.gff \
                  $ANNOGESIC_FOLDER/output/sORFs/gffs/best_candidates/${SEQ_ID}_sORF.gff \
                  $ANNOGESIC_FOLDER/output/riboswitches/gffs/${SEQ_ID}_riboswitch.gff \
                  $ANNOGESIC_FOLDER/output/RNA_thermometers/gffs/${SEQ_ID}_RNA_thermometer.gff \
                  $ANNOGESIC_FOLDER/output/crisprs/gffs/best_candidates/${SEQ_ID}_CRISPR.gff"

    $ANNOGESIC_PATH \
        merge_features \
        -a $ANNOGESIC_FOLDER/output/transcripts/gffs/${SEQ_ID}_transcript.gff \
        -of $ALL_FEATURES \
        -op ${SEQ_ID} \
        -s RefSeq \
        -pj $ANNOGESIC_FOLDER
}

main
