main(){
    PATH_FILE=$(pwd)
    PYTHON_PATH=python3
    ANNOGESIC_PATH=annogesic #### specify the path of annogesic
    ANNOGESIC_FOLDER=ANNOgesic_NC_000913_RNA-Seq
    FTP_SOURCE=ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/bacteria/Escherichia_coli/latest_assembly_versions/GCF_000005845.2_ASM584v2/
    READEMPTION_PATH=reademption #### specify the path of reademption
    READEMPTION_FOLDER=READemption_NC_000913_RNA-Seq
    SEQ_ID=NC_000913.3
    WIG_FOLDER=$ANNOGESIC_FOLDER/input/wigs/fragment
    LIBS="$WIG_FOLDER/SRR794848_forward.wig:frag:1:a:+ \
$WIG_FOLDER/SRR794848_reverse.wig:frag:1:a:- \
$WIG_FOLDER/SRR794849_forward.wig:frag:1:b:+ \
$WIG_FOLDER/SRR794849_reverse.wig:frag:1:b:- \
$WIG_FOLDER/SRR794850_forward.wig:frag:1:c:+ \
$WIG_FOLDER/SRR794850_reverse.wig:frag:1:c:-"

##### Please remove # before the commands that you want to run
##### Please install ANNOgesic properly or use a Docker container

    set_up_analysis_folder
#    get_wig_and_read_files
#    get_input_files    
#    run_read_alignment
#    build_coverage_files
#    Transcript_detection
#    Terminator_prediction
#    operon_detection
#    sRNA_detection
#    sRNA_target
#    crispr
#    merge_features
}


set_up_analysis_folder(){
    if ! [ -d $ANNOGESIC_FOLDER ]
    then
        $ANNOGESIC_PATH create \
	-pj $ANNOGESIC_FOLDER
    fi

    ##### Please install READemption properly or use a Docker container

    if ! [ -d $READEMPTION_FOLDER ]
    then
        $READEMPTION_PATH create $READEMPTION_FOLDER
    fi
}

get_wig_and_read_files(){
    #### Download and install sratoolkit. If sratoolkit has been installed, you can skip this step.
    wget http://ftp-trace.ncbi.nlm.nih.gov/sra/sdk/2.5.2/sratoolkit.2.5.2-ubuntu64.tar.gz
    tar -zxvf sratoolkit.2.5.2-ubuntu64.tar.gz
    rm sratoolkit.2.5.2-ubuntu64.tar.gz

    #### transfer SRA files to fasta files. Please specify the path of fastq_dump
    for SRA in SRR794848 SRR794849 SRR794850
    do
        wget ftp://ftp-trace.ncbi.nlm.nih.gov/sra/sra-instant/reads/ByStudy/sra/SRP/SRP019/SRP019956/$SRA/$SRA.sra
        ./sratoolkit.2.5.2-ubuntu64/bin/fastq-dump.2.5.2 --fasta $SRA.sra
        rm $SRA.sra
    done
    cp *.fasta $READEMPTION_FOLDER/input/reads
    mv *.fasta $ANNOGESIC_FOLDER/input/reads
}

get_input_files(){
    $ANNOGESIC_PATH \
	get_input_files \
	-F $FTP_SOURCE \
	-g \
	-f \
	-e \
	-k \
	-p \
	-r \
	-pj $ANNOGESIC_FOLDER
    cp $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa $READEMPTION_FOLDER/input/reference_sequences
    cp $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff $READEMPTION_FOLDER/input/annotations
}

run_read_alignment(){
   # * use --split if you need run circRNA prediction
   # * with -r the remapper is used - might take significantly longer but should be worth it
    $READEMPTION_PATH \
        align \
        -r \
        -p 24 \
        -a 95 \
        -l 12 \
        --poly_a_clipping \
        --progress \
        $READEMPTION_FOLDER
}

build_coverage_files(){
    $READEMPTION_PATH \
        coverage \
        -p 24 \
        $READEMPTION_FOLDER
    cp $READEMPTION_FOLDER/output/coverage/coverage-raw/* $ANNOGESIC_FOLDER/input/wigs/fragment
}

Transcript_detection(){
    $ANNOGESIC_PATH \
        transcript \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -fl $LIBS \
        -rf all_2 \
	-cf gene CDS \
        -pj $ANNOGESIC_FOLDER
}

Terminator_prediction(){
    $ANNOGESIC_PATH \
        terminator \
        -f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -a $ANNOGESIC_FOLDER/output/transcripts/gffs/${SEQ_ID}_transcript.gff \
        -fl $LIBS \
        -rf all_2 \
        -pj $ANNOGESIC_FOLDER
}

operon_detection(){
    $ANNOGESIC_PATH \
        operon \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -a $ANNOGESIC_FOLDER/output/transcripts/gffs/${SEQ_ID}_transcript.gff \
        -e $ANNOGESIC_FOLDER/output/terminators/gffs/best_candidates/${SEQ_ID}_term.gff \
        -pj $ANNOGESIC_FOLDER
}

sRNA_detection(){
    #### If you have no sRNA database, you can use BSRD.
    wget -cP $ANNOGESIC_FOLDER/input/databases/ https://raw.githubusercontent.com/Sung-Huan/ANNOgesic/master/database/sRNA_database_BSRD.fa
    #### If you have no nr databse, please download it.
    wget -cP $ANNOGESIC_FOLDER/input/databases/ ftp://ftp.ncbi.nih.gov/blast/db/FASTA/nr.gz
    gunzip $ANNOGESIC_FOLDER/input/databases/nr.gz
    mv $ANNOGESIC_FOLDER/input/databases/nr $ANNOGESIC_FOLDER/input/databases/nr.fa


    $ANNOGESIC_PATH \
        srna \
        -d blast_srna sec_str blast_nr \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -a $ANNOGESIC_FOLDER/output/transcripts/gffs/${SEQ_ID}_transcript.gff \
        -f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -e $ANNOGESIC_FOLDER/output/terminators/gffs/best_candidates/${SEQ_ID}_term.gff \
        -m \
        -u \
	-cs \
        -sf \
        -nf \
        -nd $ANNOGESIC_FOLDER/input/databases/nr \
	-sd $ANNOGESIC_FOLDER/input/databases/sRNA_database_BSRD \
	-fl $LIBS \
        -rf all_2 \
        -pj $ANNOGESIC_FOLDER
}

sRNA_target(){
    $ANNOGESIC_PATH \
        srna_target \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -s $ANNOGESIC_FOLDER/output/sRNAs/gffs/best_candidates/${SEQ_ID}_sRNA.gff \
        -q all \
        -p IntaRNA RNAplex RNAup \
	-mi H \
        -pj $ANNOGESIC_FOLDER
}

crispr(){
    $ANNOGESIC_PATH \
        crispr \
        -g $ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
        -f $ANNOGESIC_FOLDER/input/references/fasta_files/${SEQ_ID}.fa \
        -pj $ANNOGESIC_FOLDER
}

merge_features(){
    ALL_FEATURES="$ANNOGESIC_FOLDER/input/references/annotations/${SEQ_ID}.gff \
                  $ANNOGESIC_FOLDER/output/terminators/gffs/best_candidates/${SEQ_ID}_term.gff \
                  $ANNOGESIC_FOLDER/output/sRNAs/gffs/best_candidates/${SEQ_ID}_sRNA.gff \
                  $ANNOGESIC_FOLDER/output/crisprs/gffs/best_candidates/${SEQ_ID}_CRISPR.gff"

    $ANNOGESIC_PATH \
        merge_features \
        -a $ANNOGESIC_FOLDER/output/transcripts/gffs/${SEQ_ID}_transcript.gff \
        -of $ALL_FEATURES \
        -op ${SEQ_ID} \
        -s RefSeq \
        -pj $ANNOGESIC_FOLDER
}

main
